"""
    常量
"""
from ..core.datastructures import ImmutableList
from ..core.datastructures import ImmutableDict
from ..sparrow_distribute.constants import DistributeStatus


class StoreNumberInfo(object):
    """ 仓库信息及对应发货单状态 """
    SERVICE_STORE_NUMBER = "B3"
    SERVICE_STORE_NAME = "B3仓库"
    SERVICE_DESK_NUMBER = "ZT"
    SERVICE_DESK_NAME = "自提点"

    DISTRIBUTE_STATUS_STORE_INFO_MAP = {
        DistributeStatus.SERVICE_DESK: {
            'name': SERVICE_DESK_NAME,
            'number': SERVICE_DESK_NUMBER,
            'code': DistributeStatus.SERVICE_DESK
        },
        DistributeStatus.SERVICE_STORE: {
            'name': SERVICE_STORE_NAME,
            'number': SERVICE_STORE_NUMBER,
            'code': DistributeStatus.SERVICE_STORE
        }
    }

    STORE_NUMBER_TO_NAME_MAP = {d['number']: d['name'] for d in DISTRIBUTE_STATUS_STORE_INFO_MAP.values()}
    STORE_NUMBER_TO_CODE_MAP = {d['number']: d['code'] for d in DISTRIBUTE_STATUS_STORE_INFO_MAP.values()}
    STORE_CODE_TO_NAME_MAP = {d['code']: d['name'] for d in DISTRIBUTE_STATUS_STORE_INFO_MAP.values()}
    STORE_CODE_TO_NUMBER_MAP = {d['code']: d['number'] for d in DISTRIBUTE_STATUS_STORE_INFO_MAP.values()}


class InventoryDetailResult(object):
    """
        发货单盘点结果
    """
    # 盘盈
    PROFIT = 'profit'
    # 盘亏
    LOSS = 'loss'
    # 持平
    BALANCE = 'balance'
    # 库内盘盈
    DISLOCATION = 'dislocation'

    # INVENTORY_DETAIL_RESULT_CHOICES = (
    #     (PROFIT, '盘盈'),
    #     (LOSS, '盘亏'),
    #     (BALANCE, '持平'),
    #     (DISLOCATION, '库内差异')
    # )

    CHECKED_RESULT_MAP = ImmutableDict({
        PROFIT: "盘盈",
        LOSS: "盘亏",
        BALANCE: "盘平",
        DISLOCATION: "库内差异"
    })

    UNBALANCE = [PROFIT, LOSS]
    UNBALANCE_MID = [PROFIT, LOSS, DISLOCATION]
    ACTUAL_CHECKED_RESULT = [BALANCE, PROFIT, DISLOCATION]

    DEALED_MESSAGE_MAP = ImmutableDict({
        PROFIT: "核销盘盈差异",
        LOSS: "核销盘亏差异",
        DISLOCATION: "核销库内位置错误差异",
    })


class InventoryDealedType:
    INTERIOR = 'interior'  # 库内核销
    EXTERIOR = 'exterior'  # 库外核销

    DealedTypeMap = ImmutableDict({
        INTERIOR: "库内核销",
        EXTERIOR: "库外核销"
    })


class InventoryMainStatus:
    """ 盘点记录状态 """
    INIT = "init"  # 初始
    COMPLETED = "completed"  # 已完成
    CLOSED = "closed"  # 已关闭


class InventoryHelpStatus:
    """ 助力盘点的状态 """
    PENDING = "pending"  # 待领取
    ACCESSED = "accessed"  # 已领取
    CHECKING = "checking"  # 盘点中
    COMPLETED = "completed"  # 已完成
    CLOSED = "closed"  # 已关闭, 由关闭盘点记录触发

    InventoryHelpStatusShowDict = ImmutableDict({
        PENDING: "待盘点",
        ACCESSED: "待盘点",
        CHECKING: "盘点中",
        COMPLETED: "盘点完成",
        CLOSED: "已关闭",
    })

    # 盘点中的状态
    CHECKING_STATUS = ImmutableList([ACCESSED, CHECKING])


class ShelfType(object):
    NORMAL = 'NORMAL'  # 常规货架
    INSTANT = 'INSTANT'  # 临时货架
    INIT = 'INIT'  # 货架管理上架时数据升级使用
    MERGED = 'MERGED'  # 系统默认合单后用于放发货单的位置, 这些合单后的发货单会直接打包发货, 所以没必要分配其他货位, 提高客服操作效率
    PICKUP = 'PICKUP'

    NORMAL_NAME = "固定货架"
    INSTANT_NAME = "临时货架"

    VALID_SHELF_TYPES_TO_CREATE = frozenset([NORMAL, INSTANT])
    VALID_SHELF_TYPES_TO_VIEW = frozenset([NORMAL, INSTANT, MERGED, PICKUP, INIT])

    SHELF_TYPE_DICT = {
        NORMAL: NORMAL_NAME,
        INSTANT: INSTANT_NAME
    }


class StringFmt(object):
    # 盘点序列号
    INVENTORY_NUMBER = '{}%y%m%d%H%M%S'
    # 固定货架编号格式
    SHELF_NORMAL_NUMBER = '{store_number}-{serial_number:0>3d}'
    # 临时货架编号格式
    SHELF_INSTANT_NUMBER = '{store_number}-LS{serial_number:0>3d}'
    # 固定货架货位编号格式
    SEAT_NORMAL_NUMBER = '{shelf_serial_number:0>3d}-{row_number}{column_number:0>3d}-{store_number}'
    # 临时货架货位编码格式
    SEAT_INSTANT_NUMBER = 'LS{shelf_serial_number:0>3d}-A000-{store_number}'

    SEAT_NUMBER_FMT_MAP = {
        ShelfType.NORMAL: SEAT_NORMAL_NUMBER,
        ShelfType.INSTANT: SEAT_INSTANT_NUMBER
    }

    SHELF_NUMBER_FMT_MAP = {
        ShelfType.NORMAL: SHELF_NORMAL_NUMBER,
        ShelfType.INSTANT: SHELF_INSTANT_NUMBER
    }

    # 货架管理上架前, 对所有在库发货单初始化库存信息, 使用该货架
    SHELF_INIT_NUMBER = '{store_number}-INIT'
    SEAT_INIT_NUMBER = 'INIT-A000-{store_number}'
    SHELF_MERGED_NUMBER = '{store_number}-888'
    SEAT_MERGED_NUMBER = '888-A000-{store_number}'
    SHELF_PICKUP_NUMBER = '{store_number}-666'
    SEAT_PICKUP_NUMBER = '666-A000-{store_number}'


ROW_NUMBERS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'


class ShelfSeatStatus(object):
    ON = 'ON'  # 正常
    OFF = 'OFF'  # 废弃
    INIT = 'INIT'  # 货架管理上架时数据升级使用


# 助力时分布式锁
ACCESS_HELPS_LOCK_KEY_FMT = "SparrowOrderDisstorageHelpsAccessLock_{main_id}_{shelf_id}"
ACCESS_HELPS_LOCK_EXPIRE = 10


class FlashInventoryConst(object):

    SERVICE_STOP_INVENTORY = "service_stop_inventory"

    FlashInventory_code2name_DICT = {
        SERVICE_STOP_INVENTORY: "滞留仓盘点"
    }

    FlashInventory_code2sql_DICT = {
        SERVICE_STOP_INVENTORY: """ select distinct np.shipping_number
from sparrow_distribute_distribute dis
join sparrow_distribute_expressrela rela on dis.id=rela.distribute_id
join sparrow_shipping_newexpressorder np on np.id=rela.newexpressorder_id
where dis.distribute_status='service_stop' """
    }



import datetime

from django.db import models

from ..core.base_models import BaseModelTimeAndDeleted
from ..core.formats import PARCEL_NUMBER_FMT
from ..core.numbers import getRandomNumSet
from ..core.common_utils import get_uuid4_hex
from .constants import TransferStatus
from .constants import ParcelStatus
from . import APP_LABEL

__all__ = ['Parcel', 'Footprint', 'Transfer', 'TransferDetail']


class Parcel(BaseModelTimeAndDeleted):
    """ 包裹档案 """
    id = models.CharField(max_length=32, primary_key=True, default=get_uuid4_hex, editable=False)
    shipping_number = models.CharField("包裹运单号", max_length=32, db_index=True)
    express_code = models.CharField("快递公司代码", max_length=32, db_index=True, null=True)
    express_name = models.CharField("快递公司名称", max_length=32, null=True)
    parcel_status = models.CharField("包裹状态", max_length=32, db_index=True, default=ParcelStatus.INIT)
    in_time = models.DateTimeField("包裹入系统时间", null=True)
    in_user_id = models.CharField("包裹入系统操作人", max_length=32, null=True)
    in_user_name = models.CharField("包裹入系统操作人", max_length=32, null=True)
    in_user_role = models.CharField("包裹入系统操作人角色", max_length=16, null=True)
    out_time = models.DateTimeField("包裹出系统时间", null=True)
    out_user_id = models.CharField("包裹出系统操作人", max_length=32, null=True)
    out_user_name = models.CharField("包裹出系统操作人", max_length=32, null=True)
    out_user_role = models.CharField("包裹出系统操作人角色", null=True, max_length=16)
    # 包裹有可能来源于 发货单打包发货、提货机打包、退货
    source = models.CharField("包裹入系统来源", max_length=16, db_index=True)

    class Meta:
        app_label = APP_LABEL


class Footprint(BaseModelTimeAndDeleted):
    """ 包裹操作日志 """
    id = models.CharField(max_length=32, primary_key=True, default=get_uuid4_hex, editable=False)
    parcel_id = models.CharField(max_length=32, editable=False, db_index=True)
    shipping_number = models.CharField("包裹运单号", max_length=32, db_index=True)
    express_code = models.CharField("快递公司代码", max_length=32, null=True)
    express_name = models.CharField("快递公司名称", max_length=32, null=True)
    event = models.CharField('事件', max_length=128, blank=True, null=True, db_index=True)
    event_sponsor = models.CharField('事件发起人', max_length=255, blank=True, null=True)
    event_sponsor_name = models.CharField('事件发起人昵称', max_length=255, blank=True, null=True)
    event_time = models.DateTimeField("Event Happened Time", blank=True, null=True, auto_now=True)
    from_status = models.CharField('起始状态', max_length=128, blank=True, null=True, db_index=True)
    to_status = models.CharField('结果状态', max_length=128, blank=True, null=True, db_index=True)
    message = models.CharField('信息', max_length=16384, blank=True, null=True)
    message_detail = models.CharField('开发看的信息', max_length=2048, blank=True, null=True)
    message_level = models.PositiveIntegerField("Log Level", default=1, db_index=True)

    class Meta:
        app_label = APP_LABEL


def get_sparrow_parcel_number():
    '''
    发货单number生成规则， 年月日时分秒+"-"+6位随机数，比如20200513123086-564734
    '''
    now = datetime.datetime.now()
    random_set = getRandomNumSet(6)
    number = now.strftime(PARCEL_NUMBER_FMT).format(rand=random_set)
    return number


class Transfer(BaseModelTimeAndDeleted):
    """ 交接档案表 """
    id = models.CharField(max_length=32, primary_key=True, default=get_uuid4_hex, editable=False)
    transfer_number = models.CharField("交接单号", max_length=32, unique=True, default=get_sparrow_parcel_number)
    express_code = models.CharField("快递公司代码", max_length=32)
    express_name = models.CharField("快递公司名称", max_length=32)
    onshelf_user_id = models.CharField("上架人信息", max_length=32, db_index=True)
    onshelf_user_name = models.CharField("上架人信息", max_length=32)
    onshelf_time = models.DateTimeField("上架时间")
    transfer_status = models.CharField("交接状态", max_length=16, db_index=True, default=TransferStatus.ACTIVE)
    recall_user_id = models.CharField("召回人信息", max_length=32, db_index=True, null=True)
    recall_user_name = models.CharField("召回人信息", max_length=32, null=True)
    recall_time = models.DateTimeField("召回时间", null=True)
    shelf_id = models.CharField("货架ID", max_length=32)
    shelf_number = models.CharField("货架编号", max_length=32)
    parcel_cnt_initial = models.PositiveIntegerField("包裹初始数量")

    class Meta:
        app_label = APP_LABEL

    @property
    def transfer_details(self):
        return TransferDetail.objects.filter(transfer_id=self.id)


class TransferDetail(BaseModelTimeAndDeleted):
    """ 交接明细表 """
    id = models.CharField(max_length=32, primary_key=True, default=get_uuid4_hex, editable=False)
    transfer_id = models.CharField("交接记录ID", max_length=32, db_index=True)
    transfer_number = models.CharField("交接单号", max_length=32, db_index=True)
    parcel_id = models.CharField(max_length=32, db_index=True)
    shipping_number = models.CharField("包裹运单号", max_length=32, db_index=True)
    express_code = models.CharField("快递公司代码", max_length=32, db_index=True, null=True)
    express_name = models.CharField("快递公司名称", max_length=32, null=True)
    # material_code varchar(12) 物料编码
    material_code = models.CharField("物料编码", max_length=12, null=True)
    # material_name varchar(12) 物料名称
    material_name = models.CharField("物料名称", max_length=12, null=True)

    class Meta:
        app_label = APP_LABEL


class Material(BaseModelTimeAndDeleted):
    """ 物料档案 """
    # name varchar(12) 名称
    name = models.CharField("物料名称", max_length=12, null=True)
    # code varchar(10) 编码，编码格式：B-001
    code = models.CharField("物料编码", max_length=12, null=True)
    # type varchar(10) 类型
    type = models.CharField("类型", max_length=10, null=True)
    # description varchar(50) 描述
    description = models.CharField("描述", max_length=50, null=True)
    # spec varchar(20) 规格
    spec = models.CharField("规格", max_length=20, null=True)
    # model varchar(20) 型号
    model = models.CharField("型号", max_length=20, null=True)
    # 排序规则
    seq = models.IntegerField("排序", default=0)

    class Meta:
        app_label = APP_LABEL
        


class MaterialOptRecord(BaseModelTimeAndDeleted):
    """ 物料档案 """
    # material_name varchar(12) 物料名称
    material_name = models.CharField("物料名称", max_length=12, null=True)
    # material_code varchar(12) 物料编码
    material_code = models.CharField("物料编码", max_length=12, null=True)
    # opt varchar(12)  （出库/入库）
    opt = models.CharField("操作类型", max_length=12, null=True)
    # opt_user_id varchar(12) 操作人ID
    opt_user_id = models.CharField("操作人ID", max_length=40, null=True)
    # opt_user_name  varchar(12) 操作人名称
    opt_user_name = models.CharField("操作人名称", max_length=32, null=True)
    # quantity 数量 int（出库/入库都是正整数）
    quantity = models.IntegerField("数量", default=0)
    

    class Meta:
        app_label = APP_LABEL




from ..core.constants import ShippingPartnerCodes as Partner

PARCEL_ERROR_CODE_PREFIX = 249000


class ParcelStatus:
    """ 包裹状态 """
    INIT = 'init'  # 初始   待上架
    CANCEL = 'cancel'  # 取消打包
    CHECKING = 'checking'  # 上架待揽收
    CHECKED = 'checked'  # 已揽收

    PARCEL_STAUTS_CHOICES = {
        INIT: "待上架",
        CANCEL: "取消打包",
        CHECKING: "待揽收",
        CHECKED: "已揽收"
    }


class ParcelEventConstKey:
    FROM_STATUS_LIST_KEY = 'from_status_list'
    TO_STATUS_KEY = 'to_status'
    TO_STATE_MAP_KEY = 'to_state_map'


class ParcelEventConst:
    NEW_ADD = 'new_add'
    CANCEL = 'cancel'  # 取消打包
    CHECKING = 'checking'  # 上架
    CHECKED = 'checked'  # 揽收
    TRANSFER_INACTIVE = 'transfer_inactive'  # 提交召回
    BULK_CHECKING = 'bulk_checking'  # 批量上架

    FROM_TO_MAP = {
        NEW_ADD: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.INIT, ParcelStatus.CANCEL,
            ],
            ParcelEventConstKey.TO_STATUS_KEY: ParcelStatus.INIT
        },
        CANCEL: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.INIT, ParcelStatus.CHECKING
            ],
            ParcelEventConstKey.TO_STATUS_KEY: ParcelStatus.CANCEL
        },
        CHECKING: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.INIT
            ],
            ParcelEventConstKey.TO_STATUS_KEY: ParcelStatus.CHECKING
        },
        BULK_CHECKING: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.INIT
            ],
            ParcelEventConstKey.TO_STATUS_KEY: ParcelStatus.CHECKING
        },
        CHECKED: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.CHECKING
            ],
            ParcelEventConstKey.TO_STATUS_KEY: ParcelStatus.CHECKED
        },
        TRANSFER_INACTIVE: {
            ParcelEventConstKey.FROM_STATUS_LIST_KEY: [
                ParcelStatus.CHECKING, ParcelStatus.CANCEL, ParcelStatus.INIT
            ],
            ParcelEventConstKey.TO_STATUS_KEY: None
        }
    }

    EVENT_TO_BUTTON_MAP = {
        NEW_ADD: "包裹打包",
        CANCEL: "包裹取消打包",
        CHECKING: "包裹上架",
        BULK_CHECKING: "批量上架",
        CHECKED: "包裹揽收",
        TRANSFER_INACTIVE: "包裹召回",
    }


class ParcelSource:
    ONLINE_SOURCE = 'online'  # 在线发货
    AFTERSALE_RETURN = 'afs_return'  # 退货寄回
    EXCHANGE_RETURN = 'ex_return'  # 换货寄回


class TransferStatus:
    """ 交接记录状态 """
    ACTIVE = "active"  # 已上架
    INACTIVE = 'inactive'  # 已召回

    TRANSFER_STAUTS_CHOICES = {
        ACTIVE: "已上架",
        INACTIVE: "已召回",
    }


class TransferEventConstKey:
    FROM_STATUS_LIST_KEY = 'from_status_list'
    TO_STATUS_KEY = 'to_status'
    TO_STATE_MAP_KEY = 'to_state_map'


class TransferEventConst:
    NEW_ADD = 'new_add'
    INACTIVE = 'inactive'

    FROM_TO_MAP = {
        NEW_ADD: {
            TransferEventConstKey.FROM_STATUS_LIST_KEY: [
                TransferStatus.ACTIVE,
            ],
            TransferEventConstKey.TO_STATUS_KEY: None,
        },
        INACTIVE: {
            TransferEventConstKey.FROM_STATUS_LIST_KEY: [
                TransferStatus.ACTIVE,
            ],
            TransferEventConstKey.TO_STATUS_KEY: TransferStatus.INACTIVE,
        }
    }

    EVENT_TO_BUTTON_MAP = {
        NEW_ADD: "建立交接档案",
        INACTIVE: "交接档案召回",
    }


class ExpressName:
    """ 包裹管理中给前端用的快递公司 """

    SF = 'sf'
    YZ = 'yz'
    ZTO = 'zto'

    CODE_2_system_codes = {
        SF: [Partner.SF, Partner.SFTCJS],
        YZ: [Partner.YZPY],
        ZTO: [Partner.ZTO]
    }

    CODE_2_NAME_MAP = {
        SF: "顺丰",
        YZ: "邮政",
        ZTO: "中通",
    }

    EXPRESS_NAME_INFOS = [
        {
            "name": CODE_2_NAME_MAP[SF],
            "code": SF,
        },
        {
            "name": CODE_2_NAME_MAP[YZ],
            "code": YZ,
        },
        {
            "name": CODE_2_NAME_MAP[ZTO],
            "code": ZTO,
        },
    ]





class MemberLevel(object):
    '''会员级别'''
    MEMBER_ALL = "ALL"
    MEMBER_L1 = 'MEMBER_L1'  # 银卡会员
    MEMBER_L2 = "MEMBER_L2"  # 金卡会员
    MEMBER_L0 = "MEMBER_L0"  # 普卡会员


class PayType(object):
    '''支付类型'''
    CASH = "1"
    CHECK = "2"
    HG_DISCOUNT = "4"
    WECHAT = "17"
    ALIPAY = "18"
    C_COUPON = "13"
    A_COUPON = "10"
    D_COUPON = "15"
    GIFT_COUPON = "16"
    # 待拆分的
    TO_SPLIT = "888"
    # 待被决定的
    TO_PAY = "999"
    # 现金类支付方式
    CASH_PAY_TYPE_VALUE = ("1", "17", "18", "2")
    # 现金类支付choise
    CASH_PAY_TYPE_CHOICE = (
        (WECHAT, "微信支付"),
        (ALIPAY, "支付宝支付"),
    )
    PAYMENT_TYPE_CHOICES = (
        (CASH, "现金"),
        (WECHAT, "微信支付"),
        (ALIPAY, "支付宝支付"),
        (A_COUPON, "A券"),
        (C_COUPON, "C券"),
        (D_COUPON, "钱包"),
        (GIFT_COUPON, "礼券"),
        (TO_SPLIT, "待拆分"),
        (TO_PAY, "待支付"),
    )


class PaymentPayer(object):
    '''费用承担方'''
    HG = "HG"  # 汉光
    SHOP = "SHOP"  # 专柜
    BAISHI = "baishi"  # 百世

    PAYER_CHOICES = (
        (HG, "汉光"),
        (SHOP, "专柜"),
        (BAISHI, "百世"),
    )


class LogLevel(object):
    INFO = 'info'
    WARNING = 'warning'
    ERROR = 'error'
    SYSTEM = 'system'


class ShippingPartnerCodes(object):
    '''ExpressPartner里面要用到的code'''
    # 顺丰
    SF = "sf"
    # 顺丰同城急送
    SFTCJS = "sftcjs"
    # 圆通
    YTO = "yto"
    # 中通
    ZTO = "zto"
    # 申通
    STO = "sto"
    # EMS
    EMS = "ems"
    # 邮政
    YZPY = "yzpy"
    # 韵达
    YD = "yd"
    # 宅急送
    ZJS = "zjs"
    # 京东
    JD = "jd"
    # 德邦
    DBL = "dbl"
    # 百世快递
    HTKY = "htky"
    # 天天
    HHTT = "hhtt"
    # 其它
    OTHERS = "others"
    # 专柜代发
    SHOP = "shop"
    # 闪送
    SS = "ss"


# 快递公司代号与中文对照
ShippingPartnerCodes_DICT = {
    ShippingPartnerCodes.SF: "顺丰",
    ShippingPartnerCodes.SFTCJS: "顺丰同城急送",
    ShippingPartnerCodes.YTO: "圆通",
    ShippingPartnerCodes.ZTO: "中通",
    ShippingPartnerCodes.STO: "申通",
    ShippingPartnerCodes.EMS: "EMS",
    ShippingPartnerCodes.YZPY: "邮政",
    ShippingPartnerCodes.YD: "韵达",
    ShippingPartnerCodes.ZJS: "宅急送",
    ShippingPartnerCodes.JD: "京东",
    ShippingPartnerCodes.DBL: "德邦",
    ShippingPartnerCodes.HTKY: "百世快递",
    ShippingPartnerCodes.HHTT: "天天",
    ShippingPartnerCodes.OTHERS: "其它",
    ShippingPartnerCodes.SHOP: "专柜代发",
    ShippingPartnerCodes.SS: "闪送",
}


class UserRole(object):
    '''
    ### 用户角色 ###
    '''
    # 客户
    CUSTOMER = "customer"
    # 客服
    CUSSERVICE = "cusservice"
    # 导购
    GUIDE = "guide"
    # 麻雀自动
    AUTO = "auto"
    # 高级客服
    SUPER_CUSSERVICE = "supercusservice"
    # 楼层
    LOUCENG = "louceng"
    # 开发
    DEV = "dev"

    USERROLE_CHOICES = (
        (CUSTOMER, "客户"),
        (CUSSERVICE, "客服"),
        (GUIDE, "导购"),
        (AUTO, "麻雀自动"),
        (SUPER_CUSSERVICE, "高级客服"),
        (LOUCENG, "楼层"),
        (DEV, "开发"),
    )
    USERROLE_DICT = dict(USERROLE_CHOICES)

    # 扶摇上权限管理角色 role_code 和 常量 role 的对应关系
    ROLE_CODE_2_ROLE = {
        'dev': DEV,
        's_customer': SUPER_CUSSERVICE,
        'j_customer': CUSSERVICE,
        'louceng': LOUCENG,
        'guide': GUIDE
    }
    # 扶摇上权限管理角色的权重顺序, 从高到低
    ROLE_WEIGHT = ['dev', 's_customer', 'j_customer', 'louceng', 'guide']





class ImmutableList(tuple):
    """
    不可变列表
    """

    def __new__(cls, *args, warning='ImmutableList object is immutable.', **kwargs):
        self = tuple.__new__(cls, *args, **kwargs)
        self.warning = warning
        return self

    def complain(self, *wargs, **kwargs):
        if isinstance(self.warning, Exception):
            raise self.warning
        else:
            raise AttributeError(self.warning)

    # All list mutation functions complain.
    __delitem__ = complain
    __delslice__ = complain
    __iadd__ = complain
    __imul__ = complain
    __setitem__ = complain
    __setslice__ = complain
    append = complain
    extend = complain
    insert = complain
    pop = complain
    remove = complain
    sort = complain
    reverse = complain


class ImmutableDict(dict):

    def NotImplemented(self, *args, **kw):
        raise ValueError("Read Only dict proxy")

    popitem = update = clear = pop = __setitem__ = __delitem__ = NotImplementedError
